@@ -68,43 +68,18 @@ static gboolean gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment);
 static GstFlowReturn gst_mms_create (GstPushSrc * psrc, GstBuffer ** buf);
 
 static gboolean gst_mms_uri_set_uri (GstURIHandler * handler,
-    const gchar * uri);
+    const gchar * uri, GError ** error);
 
-static void
-gst_mms_urihandler_init (GType mms_type)
-{
-  static const GInterfaceInfo urihandler_info = {
-    gst_mms_uri_handler_init,
-    NULL,
-    NULL
-  };
-
-  g_type_add_interface_static (mms_type, GST_TYPE_URI_HANDLER,
-      &urihandler_info);
-}
-
-GST_BOILERPLATE_FULL (GstMMS, gst_mms, GstPushSrc, GST_TYPE_PUSH_SRC,
-    gst_mms_urihandler_init);
-
-static void
-gst_mms_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_set_details_simple (element_class, "MMS streaming source",
-      "Source/Network",
-      "Receive data streamed via MSFT Multi Media Server protocol",
-      "Maciej Katafiasz <mathrick@users.sourceforge.net>");
-
-  GST_DEBUG_CATEGORY_INIT (mmssrc_debug, "mmssrc", 0, "MMS Source Element");
-}
+#define gst_mms_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstMMS, gst_mms, GST_TYPE_PUSH_SRC,
+    G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER, gst_mms_uri_handler_init));
 
 /* initialize the plugin's class */
 static void
 gst_mms_class_init (GstMMSClass * klass)
 {
   GObjectClass *gobject_class = (GObjectClass *) klass;
+  GstElementClass *gstelement_class = (GstElementClass *) klass;
   GstBaseSrcClass *gstbasesrc_class = (GstBaseSrcClass *) klass;
   GstPushSrcClass *gstpushsrc_class = (GstPushSrcClass *) klass;
 
@@ -117,12 +92,23 @@ gst_mms_class_init (GstMMSClass * klass)
           "Host URL to connect to. Accepted are mms://, mmsu://, mmst:// URL types",
           NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /* Note: connection-speed is intentionaly limited to G_MAXINT as libmms
+   * uses int for it */
   g_object_class_install_property (gobject_class, PROP_CONNECTION_SPEED,
-      g_param_spec_uint ("connection-speed", "Connection Speed",
+      g_param_spec_uint64 ("connection-speed", "Connection Speed",
           "Network connection speed in kbps (0 = unknown)",
           0, G_MAXINT / 1000, DEFAULT_CONNECTION_SPEED,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  /* Note: connection-speed is intentionaly limited to G_MAXINT as libmms use int for it */
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
+
+  gst_element_class_set_details_simple (gstelement_class,
+      "MMS streaming source", "Source/Network",
+      "Receive data streamed via MSFT Multi Media Server protocol",
+      "Maciej Katafiasz <mathrick@users.sourceforge.net>");
+
+  GST_DEBUG_CATEGORY_INIT (mmssrc_debug, "mmssrc", 0, "MMS Source Element");
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_mms_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_mms_stop);
@@ -143,7 +129,7 @@ gst_mms_class_init (GstMMSClass * klass)
  * initialize structure
  */
 static void
-gst_mms_init (GstMMS * mmssrc, GstMMSClass * g_class)
+gst_mms_init (GstMMS * mmssrc)
 {
   mmssrc->uri_name = NULL;
   mmssrc->current_connection_uri_name = NULL;
@@ -263,7 +249,7 @@ gst_mms_prepare_seek_segment (GstBaseSrc * src, GstEvent * event,
      of the segment. */
 
   gst_segment_init (segment, seek_format);
-  gst_segment_set_seek (segment, rate, seek_format, flags, cur_type, cur,
+  gst_segment_do_seek (segment, rate, seek_format, flags, cur_type, cur,
       stop_type, stop, NULL);
 
   return TRUE;
@@ -300,8 +286,8 @@ gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment)
     return FALSE;
   }
   gst_segment_init (segment, GST_FORMAT_BYTES);
-  gst_segment_set_seek (segment, segment->rate, GST_FORMAT_BYTES,
-      segment->flags, GST_SEEK_TYPE_SET, start, GST_SEEK_TYPE_NONE,
+  gst_segment_do_seek (segment, segment->rate, GST_FORMAT_BYTES,
+      GST_SEEK_FLAG_NONE, GST_SEEK_TYPE_SET, start, GST_SEEK_TYPE_NONE,
       segment->stop, NULL);
   return TRUE;
 }
@@ -338,37 +324,32 @@ gst_mms_create (GstPushSrc * psrc, GstBuffer ** buf)
   else
     blocksize = mmsx_get_asf_packet_len (mmssrc->connection);
 
-  *buf = gst_buffer_try_new_and_alloc (blocksize);
-  if (!*buf) {
+  data = g_try_malloc (blocksize);
+  if (!data) {
     GST_ERROR_OBJECT (mmssrc, "Failed to allocate %u bytes", blocksize);
     return GST_FLOW_ERROR;
   }
 
-  data = GST_BUFFER_DATA (*buf);
-  GST_BUFFER_SIZE (*buf) = 0;
   GST_LOG_OBJECT (mmssrc, "reading %d bytes", blocksize);
   result = mmsx_read (NULL, mmssrc->connection, (char *) data, blocksize);
-
   /* EOS? */
   if (result == 0)
     goto eos;
 
+  *buf = gst_buffer_new_wrapped (data, result);
   GST_BUFFER_OFFSET (*buf) = offset;
-  GST_BUFFER_SIZE (*buf) = result;
 
   GST_LOG_OBJECT (mmssrc, "Returning buffer with offset %" G_GINT64_FORMAT
-      " and size %u", GST_BUFFER_OFFSET (*buf), GST_BUFFER_SIZE (*buf));
-
-  gst_buffer_set_caps (*buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (mmssrc)));
+      " and size %u", offset, result);
 
   return GST_FLOW_OK;
 
 eos:
   {
     GST_DEBUG_OBJECT (mmssrc, "EOS");
-    gst_buffer_unref (*buf);
+    g_free (data);
     *buf = NULL;
-    return GST_FLOW_UNEXPECTED;
+    return GST_FLOW_EOS;
   }
 }
 
@@ -496,11 +477,11 @@ gst_mms_set_property (GObject * object, guint prop_id,
   switch (prop_id) {
     case PROP_LOCATION:
       gst_mms_uri_set_uri (GST_URI_HANDLER (mmssrc),
-          g_value_get_string (value));
+          g_value_get_string (value), NULL);
       break;
     case PROP_CONNECTION_SPEED:
       GST_OBJECT_LOCK (mmssrc);
-      mmssrc->connection_speed = g_value_get_uint (value) * 1000;
+      mmssrc->connection_speed = g_value_get_uint64 (value) * 1000;
       GST_OBJECT_UNLOCK (mmssrc);
       break;
     default:
@@ -522,7 +503,7 @@ gst_mms_get_property (GObject * object, guint prop_id,
         g_value_set_string (value, mmssrc->uri_name);
       break;
     case PROP_CONNECTION_SPEED:
-      g_value_set_uint (value, mmssrc->connection_speed / 1000);
+      g_value_set_uint64 (value, mmssrc->connection_speed / 1000);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -543,25 +524,26 @@ plugin_init (GstPlugin * plugin)
 }
 
 static GstURIType
-gst_mms_uri_get_type (void)
+gst_mms_uri_get_type (GType type)
 {
   return GST_URI_SRC;
 }
 
-static gchar **
-gst_mms_uri_get_protocols (void)
+static const gchar *const *
+gst_mms_uri_get_protocols (GType type)
 {
   static const gchar *protocols[] = { "mms", "mmsh", "mmst", "mmsu", NULL };
 
-  return (gchar **) protocols;
+  return protocols;
 }
 
-static const gchar *
+static gchar *
 gst_mms_uri_get_uri (GstURIHandler * handler)
 {
   GstMMS *src = GST_MMS (handler);
 
-  return src->uri_name;
+  /* FIXME: make thread-safe */
+  return g_strdup (src->uri_name);
 }
 
 static gchar *
@@ -613,14 +595,18 @@ gst_mms_src_make_valid_uri (const gchar * uri)
 }
 
 static gboolean
-gst_mms_uri_set_uri (GstURIHandler * handler, const gchar * uri)
+gst_mms_uri_set_uri (GstURIHandler * handler, const gchar * uri,
+    GError ** error)
 {
   GstMMS *src = GST_MMS (handler);
   gchar *fixed_uri;
 
   fixed_uri = gst_mms_src_make_valid_uri (uri);
-  if (!fixed_uri && uri)
+  if (!fixed_uri && uri) {
+    g_set_error (error, GST_URI_ERROR, GST_URI_ERROR_BAD_URI,
+        "Invalid MMS URI");
     return FALSE;
+  }
 
   GST_OBJECT_LOCK (src);
   if (src->uri_name)
@@ -647,6 +633,6 @@ gst_mms_uri_handler_init (gpointer g_iface, gpointer iface_data)
  * so keep the name plugin_desc, or you cannot get your plug-in registered */
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "mms",
+    mms,
     "Microsoft Multi Media Server streaming protocol support",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
