@@ -2,6 +2,7 @@
  * Copyright (C) 2004 Wim Taymans <wim@fluendo.com>
  * Copyright (C) 2006 Tim-Philipp Müller <tim centricular net>
  * Copyright (C) 2008 Sebastian Dröge <sebastian.droege@collabora.co.uk>
+ * Copyright (C) 2011-2012 Vincent Penquerc'h <vincent.penquerch@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -123,7 +124,7 @@ gst_opus_dec_class_init (GstOpusDecClass * klass)
   gst_element_class_set_details_simple (element_class, "Opus audio decoder",
       "Codec/Decoder/Audio",
       "decode opus streams to audio",
-      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
+      "Vincent Penquerc'h <vincent.penquerch@collabora.co.uk>");
   g_object_class_install_property (gobject_class, PROP_USE_INBAND_FEC,
       g_param_spec_boolean ("use-inband-fec", "Use in-band FEC",
           "Use forward error correction if available", DEFAULT_USE_INBAND_FEC,
@@ -261,17 +262,23 @@ gst_opus_dec_parse_header (GstOpusDec * dec, GstBuffer * buf)
   const GstAudioChannelPosition *posn = NULL;
   GstMapInfo map;
 
-  g_return_val_if_fail (gst_opus_header_is_id_header (buf), GST_FLOW_ERROR);
+  if (!gst_opus_header_is_id_header (buf)) {
+    GST_ERROR_OBJECT (dec, "Header is not an Opus ID header");
+    return GST_FLOW_ERROR;
+  }
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
   data = map.data;
 
-  g_return_val_if_fail (dec->n_channels == 0
-      || dec->n_channels == data[9], GST_FLOW_ERROR);
+  if (!(dec->n_channels == 0 || dec->n_channels == data[9])) {
+    gst_buffer_unmap (buf, &map);
+    GST_ERROR_OBJECT (dec, "Opus ID header has invalid channels");
+    return GST_FLOW_ERROR;
+  }
 
   dec->n_channels = data[9];
   dec->pre_skip = GST_READ_UINT16_LE (data + 10);
-  dec->r128_gain = GST_READ_UINT16_LE (data + 14);
+  dec->r128_gain = GST_READ_UINT16_LE (data + 16);
   dec->r128_gain_volume = gst_opus_dec_get_r128_volume (dec->r128_gain);
   GST_INFO_OBJECT (dec,
       "Found pre-skip of %u samples, R128 gain %d (volume %f)",
@@ -393,17 +400,20 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
      to potentially wait for next buffer to decode a missing buffer */
   if (dec->use_inband_fec && !dec->primed) {
     GST_DEBUG_OBJECT (dec, "First buffer received in FEC mode, early out");
+    gst_buffer_replace (&dec->last_buffer, buffer);
+    dec->primed = TRUE;
     goto done;
   }
 
   /* That's the buffer we'll be sending to the opus decoder. */
-  buf = dec->use_inband_fec && dec->last_buffer ? dec->last_buffer : buffer;
+  buf = (dec->use_inband_fec
+      && gst_buffer_get_size (dec->last_buffer) >
+      0) ? dec->last_buffer : buffer;
 
-  if (buf) {
+  if (buf && gst_buffer_get_size (buf) > 0) {
     gst_buffer_map (buf, &map, GST_MAP_READ);
     data = map.data;
     size = map.size;
-
     GST_DEBUG_OBJECT (dec, "Using buffer of size %u", size);
   } else {
     /* concealment data, pass NULL as the bits parameters */
@@ -428,15 +438,18 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
   if (dec->use_inband_fec) {
     if (dec->last_buffer) {
       /* normal delayed decode */
+      GST_LOG_OBJECT (dec, "FEC enabled, decoding last delayed buffer");
       n = opus_multistream_decode (dec->state, data, size, out_data, samples,
           0);
     } else {
       /* FEC reconstruction decode */
+      GST_LOG_OBJECT (dec, "FEC enabled, reconstructing last buffer");
       n = opus_multistream_decode (dec->state, data, size, out_data, samples,
           1);
     }
   } else {
     /* normal decode */
+    GST_LOG_OBJECT (dec, "FEC disabled, decoding buffer");
     n = opus_multistream_decode (dec->state, data, size, out_data, samples, 0);
   }
   gst_buffer_unmap (outbuf, &omap);
@@ -445,6 +458,7 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
 
   if (n < 0) {
     GST_ELEMENT_ERROR (dec, STREAM, DECODE, ("Decoding error: %d", n), (NULL));
+    gst_buffer_unref (outbuf);
     return GST_FLOW_ERROR;
   }
   GST_DEBUG_OBJECT (dec, "decoded %d samples", n);
@@ -495,17 +509,16 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
     gst_buffer_unmap (outbuf, &omap);
   }
 
+  if (dec->use_inband_fec) {
+    gst_buffer_replace (&dec->last_buffer, buffer);
+  }
+
   res = gst_audio_decoder_finish_frame (GST_AUDIO_DECODER (dec), outbuf, 1);
 
   if (res != GST_FLOW_OK)
     GST_DEBUG_OBJECT (dec, "flow: %s", gst_flow_get_name (res));
 
 done:
-  if (dec->use_inband_fec) {
-    gst_buffer_replace (&dec->last_buffer, buffer);
-    dec->primed = TRUE;
-  }
-
   return res;
 
 creation_failed:
