@@ -72,10 +72,16 @@
 #define DESC_STD                              0x11
 #define DESC_IBP                              0x12
 
-#define DESC_DIRAC_TC_PRIVATE                 0xAC
-
-/* DVB tags */
+/* 19-26 Defined in ISO/IEC 13818-6 (Extensions for DSM-CC) */
 #define DESC_DVB_CAROUSEL_IDENTIFIER          0x13
+
+/* 27-44 Later additions to ISO/IEC 13818-1 */
+
+/* 45-63 ITU-T Rec. H.222.0 | ISO/IEC 13818-1 Reserved */
+
+/* 64-127 DVB tags ETSI EN 300 468
+ * (Specification for Service Information (SI) in DVB systems)
+ */
 #define DESC_DVB_NETWORK_NAME                 0x40
 #define DESC_DVB_SERVICE_LIST                 0x41
 #define DESC_DVB_STUFFING                     0x42
@@ -139,11 +145,19 @@
 #define DESC_DVB_AAC                          0x7C
 /* 0x7D and 0x7E are reserved for future use */
 #define DESC_DVB_EXTENSION                    0x7F
+
 /* 0x80 - 0xFE are user defined */
 #define DESC_AC3_AUDIO_STREAM                 0x81
 #define DESC_DTG_LOGICAL_CHANNEL              0x83    /* from DTG D-Book */
+
+/* Others */
+#define DESC_DIRAC_TC_PRIVATE                 0xAC
+
+
 /* 0xFF is forbidden */
 
+
+
 /* common for all descriptors */
 #define DESC_TAG(desc) 		(desc[0])
 #define DESC_LENGTH(desc) 	(desc[1])
@@ -279,7 +293,7 @@
 #define DESC_DVB_SATELLITE_DELIVERY_SYSTEM_orbital_position(desc)	(desc + 6)
 #define DESC_DVB_SATELLITE_DELIVERY_SYSTEM_west_east_flag(desc)	((desc[8] & 0x80) == 0x80)
 #define DESC_DVB_SATELLITE_DELIVERY_SYSTEM_polarization(desc)	((desc[8] >> 5) & 0x3)
-#define DESC_DVB_SATELLITE_DELIVERY_SYSTEM_modulation(desc)	(desc[8] & 0x1F)
+#define DESC_DVB_SATELLITE_DELIVERY_SYSTEM_modulation(desc)	(desc[8] & 0x3)
 #define DESC_DVB_SATELLITE_DELIVERY_SYSTEM_symbol_rate(desc)	(desc + 9)
 #define DESC_DVB_SATELLITE_DELIVERY_SYSTEM_fec_inner(desc)	(desc[12] & 0x0F)
 
@@ -335,8 +349,7 @@ typedef struct
 } GstMPEGDescriptor;
 
 void gst_mpegtsdesc_init_debug (void);
-GstMPEGDescriptor *gst_mpeg_descriptor_parse (guint8 * data, guint size);
-void gst_mpeg_descriptor_free                (GstMPEGDescriptor * desc);
+gboolean gst_mpeg_descriptor_parse (GstMPEGDescriptor *result, guint8 * data, guint size);
 
 guint gst_mpeg_descriptor_n_desc             (GstMPEGDescriptor * desc);
 guint8 *gst_mpeg_descriptor_find             (GstMPEGDescriptor * desc, gint tag);
