@@ -436,6 +436,8 @@ pause:
         gst_element_post_message (GST_ELEMENT_CAST (rp),
             gst_message_new_segment_done (GST_OBJECT_CAST (rp),
                 rp->segment.format, stop));
+        gst_pad_push_event (rp->srcpad,
+            gst_event_new_segment_done (rp->segment.format, stop));
       } else {
         GST_LOG_OBJECT (rp, "Sending EOS, at end of stream");
         gst_pad_push_event (rp->srcpad, gst_event_new_eos ());
@@ -864,7 +866,8 @@ gst_raw_parse_handle_seek_pull (GstRawParse * rp, GstEvent * event)
   rp->discont = TRUE;
 
   GST_LOG_OBJECT (rp, "start streaming");
-  gst_pad_start_task (rp->sinkpad, (GstTaskFunction) gst_raw_parse_loop, rp);
+  gst_pad_start_task (rp->sinkpad, (GstTaskFunction) gst_raw_parse_loop, rp,
+      NULL);
 
   GST_PAD_STREAM_UNLOCK (rp->sinkpad);
 
