@@ -155,6 +155,7 @@ gst_dvbsrc_modulation_get_type (void)
     {QAM_AUTO, "AUTO", "auto"},
     {VSB_8, "8VSB", "8vsb"},
     {VSB_16, "16VSB", "16vsb"},
+    {PSK_8, "8PSK", "8psk"},
     {0, NULL, NULL},
   };
 
@@ -521,24 +522,28 @@ gst_dvbsrc_set_property (GObject * _object, guint prop_id,
       break;
     case ARG_DVBSRC_FREQUENCY:
       object->freq = g_value_get_uint (value);
-      GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_FREQUENCY");
+      GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_FREQUENCY (%d Hz)",
+          object->freq);
       break;
     case ARG_DVBSRC_POLARITY:
     {
       const char *s = NULL;
 
       s = g_value_get_string (value);
-      if (s != NULL)
+      if (s != NULL) {
         object->pol = (s[0] == 'h' || s[0] == 'H') ? DVB_POL_H : DVB_POL_V;
-    }
-      GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_POLARITY");
+        GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_POLARITY");
+        GST_INFO_OBJECT (object, "\t%s", (s[0] == 'h'
+                || s[0] == 'H') ? "DVB_POL_H" : "DVB_POL_V");
+      }
       break;
+    }
     case ARG_DVBSRC_PIDS:
     {
       gchar *pid_string;
 
-      GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_PIDS");
       pid_string = g_value_dup_string (value);
+      GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_PIDS %s", pid_string);
       if (!strcmp (pid_string, "8192")) {
         /* get the whole ts */
         int pid_count = 1;
@@ -564,7 +569,7 @@ gst_dvbsrc_set_property (GObject * _object, guint prop_id,
         while (*pids != NULL && pid_count < MAX_FILTERS) {
           pid = strtol (*pids, NULL, 0);
           if (pid > 1 && pid <= 8192) {
-            GST_INFO_OBJECT (object, "Parsed Pid: %d\n", pid);
+            GST_INFO_OBJECT (object, "\tParsed Pid: %d", pid);
             object->pids[pid_count] = pid;
             pid_count++;
           }
@@ -588,7 +593,7 @@ gst_dvbsrc_set_property (GObject * _object, guint prop_id,
     case ARG_DVBSRC_SYM_RATE:
       object->sym_rate = g_value_get_uint (value);
       GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_SYM_RATE to value %d",
-          g_value_get_int (value));
+          object->sym_rate);
       break;
 
     case ARG_DVBSRC_BANDWIDTH:
@@ -718,7 +723,7 @@ gst_dvbsrc_close_devices (GstDvbSrc * object)
 }
 
 static gboolean
-gst_dvbsrc_open_frontend (GstDvbSrc * object)
+gst_dvbsrc_open_frontend (GstDvbSrc * object, gboolean writable)
 {
   struct dvb_frontend_info fe_info;
   const char *adapter_desc = NULL;
@@ -731,7 +736,8 @@ gst_dvbsrc_open_frontend (GstDvbSrc * object)
   GST_INFO_OBJECT (object, "Using frontend device: %s", frontend_dev);
 
   /* open frontend */
-  if ((object->fd_frontend = open (frontend_dev, O_RDWR)) < 0) {
+  if ((object->fd_frontend =
+          open (frontend_dev, writable ? O_RDWR : O_RDONLY)) < 0) {
     switch (errno) {
       case ENOENT:
         GST_ELEMENT_ERROR (object, RESOURCE, NOT_FOUND,
@@ -749,6 +755,8 @@ gst_dvbsrc_open_frontend (GstDvbSrc * object)
     return FALSE;
   }
 
+  GST_DEBUG_OBJECT (object, "Device opened, querying information");
+
   if (ioctl (object->fd_frontend, FE_GET_INFO, &fe_info) < 0) {
     GST_ELEMENT_ERROR (object, RESOURCE, SETTINGS,
         (_("Could not get settings from frontend device \"%s\"."),
@@ -759,6 +767,8 @@ gst_dvbsrc_open_frontend (GstDvbSrc * object)
     return FALSE;
   }
 
+  GST_DEBUG_OBJECT (object, "Got information about adapter : %s", fe_info.name);
+
   adapter_name = g_strdup (fe_info.name);
 
   object->adapter_type = fe_info.type;
@@ -869,7 +879,7 @@ gst_dvbsrc_finalize (GObject * _object)
 /*
  ******************************
  *                            *
- *      Plugin Realisation    *
+ *      Plugin Realization    *
  *                            *
  ******************************
  */
@@ -952,7 +962,7 @@ stopped:
     GST_DEBUG_OBJECT (object, "stop called");
     gst_buffer_unmap (buf, &map);
     gst_buffer_unref (buf);
-    return GST_FLOW_WRONG_STATE;
+    return GST_FLOW_FLUSHING;
   }
 select_error:
   {
@@ -1010,7 +1020,7 @@ gst_dvbsrc_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       /* open frontend then close it again, just so caps sent */
-      gst_dvbsrc_open_frontend (src);
+      gst_dvbsrc_open_frontend (src, FALSE);
       if (src->fd_frontend) {
         close (src->fd_frontend);
       }
@@ -1028,7 +1038,7 @@ gst_dvbsrc_start (GstBaseSrc * bsrc)
 {
   GstDvbSrc *src = GST_DVBSRC (bsrc);
 
-  gst_dvbsrc_open_frontend (src);
+  gst_dvbsrc_open_frontend (src, TRUE);
   if (!gst_dvbsrc_tune (src)) {
     GST_ERROR_OBJECT (src, "Not able to lock on to the dvb channel");
     close (src->fd_frontend);
@@ -1138,7 +1148,7 @@ gst_dvbsrc_frontend_status (GstDvbSrc * object)
   fe_status_t status = 0;
   gint i;
 
-  GST_INFO_OBJECT (object, "gst_dvbsrc_frontend_status\n");
+  GST_INFO_OBJECT (object, "gst_dvbsrc_frontend_status");
 
   if (object->fd_frontend < 0) {
     GST_ERROR_OBJECT (object,
@@ -1162,7 +1172,7 @@ gst_dvbsrc_frontend_status (GstDvbSrc * object)
 
   if (!(status & FE_HAS_LOCK)) {
     GST_INFO_OBJECT (object,
-        "Not able to lock to the signal on the given frequency.\n");
+        "Not able to lock to the signal on the given frequency.");
     return FALSE;
   } else
     return TRUE;
@@ -1189,7 +1199,7 @@ diseqc_send_msg (int fd, fe_sec_voltage_t v, struct diseqc_cmd *cmd,
   }
 
   usleep (15 * 1000);
-  GST_LOG ("diseqc: 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n", cmd->cmd.msg[0],
+  GST_LOG ("diseqc: 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x", cmd->cmd.msg[0],
       cmd->cmd.msg[1], cmd->cmd.msg[2], cmd->cmd.msg[3], cmd->cmd.msg[4],
       cmd->cmd.msg[5]);
   if (ioctl (fd, FE_DISEQC_SEND_MASTER_CMD, &cmd->cmd) == -1) {
@@ -1342,7 +1352,7 @@ gst_dvbsrc_tune (GstDvbSrc * object)
 #endif
         feparams.inversion = object->inversion;
 
-        GST_INFO_OBJECT (object, "tuning DVB-T to %d Hz\n", freq);
+        GST_INFO_OBJECT (object, "tuning DVB-T to %d Hz", freq);
         break;
       case FE_QAM:
         GST_INFO_OBJECT (object, "Tuning DVB-C to %d, srate=%d", freq,
